#!/usr/bin/env python

import os
import sys
import shutil
import json
import subprocess


def config_get(attribute):
    cmd = ['config-get', attribute]
    return subprocess.check_output(cmd).strip()  # IGNORE:E1103


def juju_log(severity, message):
    cmd = ['juju-log', '--log-level', severity, message]
    subprocess.check_call(cmd)


def json_dump_p(obj, f):
    """Calls json.dump with standard (pretty) formatting"""
    return json.dump(obj, f, sort_keys=True, indent=4, separators=(',', ': '))


def json_dumps_p(obj):
    """Calls json.dumps with standard (pretty) formatting"""
    return json.dumps(obj, sort_keys=True, indent=4, separators=(',', ': '))


def install():
    cwd = os.environ['CHARM_DIR']
    subprocess.check_call(['python', 'setup.py', 'install'], cwd='%s/files/turku-agent' % cwd)

    shutil.copyfile('files/turku-agent/turku-agent-rsyncd.conf',
                    '/etc/init/turku-agent-rsyncd.conf')
    shutil.copyfile('files/turku-agent/turku-agent.cron',
                    '/etc/cron.d/turku-agent')


def configure(juju_stored=None):
    if not juju_stored:
        if not os.path.exists('/var/lib/juju/turku-agent.json'):
            return
        with open('/var/lib/juju/turku-agent.json') as f:
            juju_stored = json.load(f)
    if not 'master_unit_name' in juju_stored:
        return

    api_url = config_get('api_url')
    api_auth = config_get('api_auth')
    if not (api_url and api_auth):
        return

    if not os.path.exists('/etc/turku-agent/config.d'):
        os.makedirs('/etc/turku-agent/config.d')

    environment_name = config_get('environment_name')
    service_name = config_get('service_name')
    unit_name = juju_stored['master_unit_name']
    if not environment_name:
        environment_name = 'Juju'
    if not service_name:
        service_name = unit_name.split('/')[0]

    config = {
        'api_url': api_url,
        'api_auth': api_auth,
        'environment_name': environment_name,
        'service_name': service_name,
        'unit_name': unit_name,
    }

    with open('/etc/turku-agent/config.d/50-juju.json', 'w') as f:
        json_dump_p(config, f)

    sources = config_get('sources')

    if not os.path.exists('/etc/turku-agent/sources.d'):
        os.makedirs('/etc/turku-agent/sources.d')

    try:
        sources_config = json.loads(sources)
    except ValueError:
        pass
    else:
        with open('/etc/turku-agent/sources.d/50-juju_turku_agent.json', 'w') as f:
            json_dump_p(sources_config, f)

    juju_log('INFO', 'Running turku-update-config')
    subprocess.call(['turku-update-config'])


def general_info_relation_changed():
    if os.path.exists('/var/lib/juju/turku-agent.json'):
        with open('/var/lib/juju/turku-agent.json') as f:
            juju_stored = json.load(f)
    else:
        juju_stored = {}

    juju_stored['master_unit_name'] = os.environ['JUJU_REMOTE_UNIT']

    with open('/var/lib/juju/turku-agent.json', 'w') as f:
        json_dump_p(juju_stored, f)

    configure(juju_stored)


if __name__ == '__main__':
    hook_name = os.path.basename(sys.argv[0])
    juju_log('INFO', 'Hook: %s' % hook_name)
    if hook_name in ('install', 'upgrade-charm'):
        install()
    elif hook_name == 'config-changed':
        configure()
    elif hook_name in ('general-info-relation-joined', 'general-info-relation-changed'):
        general_info_relation_changed()
